@@ -7,17 +7,18 @@
 """This tests Lithium's main "minimize" algorithm."""
 
 import sys
+from typing import List
 
 
-def interesting(args, _temp_prefix):
+def interesting(args: List[str], _temp_prefix: str) -> bool:
     """Interesting if the product of the numbers in the file divides the argument.
 
     Args:
-        args (str): The first parameter.
-        _temp_prefix (str): The second parameter.
+        args: Input arguments.
+        _temp_prefix: Temp directory prefix.
 
     Returns:
-        bool: True if successful, False otherwise.
+        True if successful, False otherwise.
     """
     mod = int(args[0])
     filename = args[1]
